<h3><a name="calc">CALC: Tutorial and Manual</a></h3>

         <UL>
            <li><a href="#intro">Introduction</a>
            <li><a href="#using">Using CALC</a>
            <li><a href="#const">Setting Constant Values</a>
            <li><a href="#conditions">Testing Conditions</a>
            <li><a href="#undefined">Undefined Variables</a>
            <li><a href="#control">Control Characters</a>
            <li><a href="#operations">Table of calc Operations</a>
         </UL>

<h4><a name="intro">Introduction</a></h4>

<p> <tt>calc</tt> is a program for
mathematical calculations for which you might use a hand-held calculator.
<tt>calc</tt> supplies most of the operations common to programming languages
and variables with constraint properties much like those in spreadsheets.  <p>
The arithmetical operators <tt>calc</tt> offers are 

<pre>
+   addition
-   subtraction and change-sign
*   multiplication
/   division
%   modulo division
^   exponentiation
</pre>

Arithmetical expressions can be arbitrarily complex and are generally evaluated
left to right.  That is, 

<pre>a + b - c</pre>

is the same as 

<pre>(a + b) - c</pre>

Exponentiation is evaluated before multiplication and division which are
evaluated before addition and subtraction.  For example, the expression 

<pre>a + b - c * d / e ^ 2</pre>

is parsed as 

<pre>(a + b) - ((c * d) / (e ^ 2))</pre>

This
default order of operations can be overridden by using parentheses.  <p>
<tt>calc</tt> supplies some transcendental functions: <tt>sqrt</tt>,
<tt>log</tt>, <tt>exp</tt>, and <tt>abs</tt>, and the following trigonometric
functions: <tt>sin</tt>, <tt>asin</tt>, <tt>cos</tt>, <tt>acos</tt>,
<tt>tan</tt>, and <tt>atan</tt>, for which degrees are measured in radians.
<p>

<h4><a name="using">Using CALC</a></h4>
<p>
To use <tt>calc</tt>, begin by typing 

<pre>calc</pre>

at the
command level, and <tt>calc</tt> will prompt you with 

<pre>CALC:</pre>

You can
supply inputs to <tt>calc</tt> from files specified by command line arguments.
For example, typing 

<pre>calc foo</pre>

will read from the file <tt>foo</tt> and
then ask for input from you.  Type in each of your expressions followed by
<tt>RETURN</tt> and <tt>calc</tt> will respond with how it parsed your
expression followed by the result.  In all following examples, what you would
type in is preceded by the <tt>calc</tt> prompt 

<pre>CALC:</pre>

and what
<tt>calc</tt> responds with is immediately after.  A simple calculation is:

<pre>CALC: sqrt (12^2 + 5^2)
sqrt(((12 ^ 2) + (5 ^ 2)))     = 13</pre>

<p>
Expressions can be stored by assigning them to variables.  For example you
could type: 

<pre>
CALC: pi = 22/7
(22 / 7)      = 3.14286
CALC: pi
pi          = 3.14286
</pre>

Variables can be used in expressions.  

<pre>
CALC: area = pi * r^2
(pi * (r ^ 2))      = UNDEFINED
CALC: area
area     = UNDEFINED
</pre>

<tt>area</tt>
is undefined because <tt>r</tt> has not been set.  Once <tt>r</tt> is set,
<tt>area</tt> will have a value because <tt>area</tt> is set to an equation
rather than a particular value.  This can be observed by printing all the
variables so far introduced with <tt>^V</tt>, which may have to be typed twice
as <tt>^V</tt> is used in some UNIX versions to quote characters.  

<pre>
CALC: ^V
pi       =      3.14286 = (22 / 7)
area     =    UNDEFINED = (pi * (r ^ 2))
r        =    UNDEFINED =
</pre>

The variable table is formatted so that each
variable's name is on the left, followed by its current value, followed by its
current definition.  If <tt>r</tt> is set to 5, the value of <tt>area</tt> is
now defined.  

<pre>
CALC: r = 5
5        = 5
CALC: ^V
pi       =      3.14286 = (22 / 7)
area     =      78.5714 = (pi * (r ^ 2))
r        =            5 = 5
</pre>

The effect of changing <tt>r</tt> on <tt>area</tt> can be observed because of
the way <tt>area</tt> is defined.  

<pre>
CALC: r = 2
2         = 2
CALC: area
area      = 12.5714
</pre>

<p> A special variable named <tt>$</tt> is always equal to
the most recent result printed.  <p>

<h4><a name="const">Setting Constant Values</a></h4>
<p>
Of course, there are times when you want to
set a variable to a value and not have it depend on the values of variables at
a later time.  To do this, you precede an expression with the number operator
<tt>#</tt>.  For example, 

<pre>
CALC: area2 = # area
12.5716      = 12.5716
CALC: ^V
pi       =      3.14286 = (22 / 7)
area     =      12.5716 = (pi * (r ^ 2))
r        =            2 = 2
area2    =      12.5716 = 12.5716
</pre>

<tt>area2</tt> does not depend on the variable to which it was set because the
number operator <tt>#</tt> only lets numbers through it rather than
expressions.  If <tt>area2</tt> was set without the <tt>#</tt> operator, it
would be subject to any changes in <tt>area</tt> or to any changes in variables
on which <tt>area</tt> depends.  

<pre>
CALC: area2 = area
area      = 12.5716
CALC: ^V
pi       =      3.14286 = (22 / 7)
area     =      12.5716 = (pi * (r ^ 2))
r        =            2 = 2
area2    =      12.5716 = area
</pre>

<h4><a name="conditions">Testing Conditions</a></h4>

<p>
Variables can be set based on a tested
condition.  For example, you may want a variable <tt>max</tt> to always be the
maximum of <tt>a</tt> and <tt>b</tt>.  

<pre>
CALC: max = if a > b then a else b
(if (a > b) then a else b)    = UNDEFINED
</pre>

<tt>max</tt> is undefined because
<tt>a</tt> and <tt>b</tt> have not been set.  

<pre>
CALC: a = 21
21     = 21
CALC: b = 3^3
(3 ^ 3)    = 27
CALC: max
max      = 27
CALC: a = 50
50   = 50
CALC: max
max      = 50
</pre>

The if-then-else expression allows variables to be set
based on conditions.  This condition can be made up with relational and logical
operators.  The relational operators available with <tt>calc</tt> are: 

<pre>
==  test equality
!=  test inequality
&gt;=  greater than or equal
&lt;=  less than or equal
&gt;   greater than
&lt;   less than
</pre>

while the logical operators are: 

<pre>
&amp;   and
|   or
!   not
</pre>

A more complicated expression involving these is: 

<pre>if a &gt; b &amp; b &gt; c then b</pre>

The <tt>else</tt> part of the conditional is optional, and if not present and
the condition is false, the conditional is undefined.  <p>

<h4><a name="undefined">Undefined Variables</a></h4>
<p> Variables are undefined if they have not been
set, if they depend on variables that are undefined, or if they are set to an
expression involving an illegal operation.  

<pre>
CALC: 1/0
(1 / 0)     = UNDEFINED
</pre>

You can be confident that no operations will result in <tt>calc</tt>
blowing up.  Thus you could write the equation for the roots of a quadratic
formula with the following definitions and always get reasonable answers.

<pre>
x = 0
a = b = 1
c = -1
radical = sqrt (b^2 - 4*a*c)
equation = a*x^2 + b*x + c
derivative = 2*a*x + b
root1 = (-b + radical) / (2 * a)
root2 = (-b - radical) / (2 * a)
</pre>

<p>

<h4><a name="control">Control Characters</a></h4>
<p>
Non-mathematical operations are accomplished
with control characters.  To type a control character, say CTRL-p, while you
hold down the key labeled CTRL you type a <tt>p</tt>.  This will appear as
<tt>^P</tt>.  Some control characters have special meanings, such as "stop the
program" so you must be careful with them.  On UNIX, you can avoid some
problems with control characters by typing a <tt>^V</tt> before them.  This
character removes any special meaning associated with the character immediately
following it.  So to type <tt>^P</tt> you could be extra safe and type
<tt>^V^P</tt>.  To type a <tt>^V</tt>, you may have to type it twice.
Unfortunately, these conventions are not universal.  <p> The following control
operations are available with <tt>calc</tt>.  

<pre>
^P  toggle the printing of expressions (UNIX only)
^Rf read the input from file f and return to current state
^V  print the variable table
^Wf write the variable table to file f
    (^W is a synonym for ^V)
</pre>

If you forget any
of these commands, you can type a <tt>?</tt> to get <tt>calc</tt> to remind
you.  

<h4><a name="operations">Table of calc Operations</a></h4>
<pre>
    Operator         Associativity
             Precedence             Description

       $       const      none      numerical value of previous calculation
       #a        1        none      numerical value of a
      a=b        2       right      a is set to expression b
  if a then b    3        left      if a != 0 then b else UNDEFINED
      else       4        left
      a|b        5        left      true if a or b is true
      a&amp;b        6        left      true is a and b are true
       !a        7        none      true is a is false
      a==b       8        none      true if a equals b
      a!=b       8        none      true if a is not equal b
      a&lt;b        8        none      true if a is less than b
      a&gt;b        8        none      true if a greater than b
      a&gt;=b       8        none      true if a &gt; b | a == b
      a&lt;=b       8        none      true if a &lt; b | a == b
      a+b        9        left      a plus b
      a-b        9        left      a minus b
      a*b        10       left      a times b
      a/b        10       left      a divided by b
      a%b        10       left      a modulo b
      a^b        11      right      a to the b
       -a        12       none      change sign
     abs(a)      12       none      absolute value
     exp(a)      12       none      e to the power a
     log(a)      12       none      natural logarithm of a
    sqrt(a)      12       none      square root of a
     sin(a)      12       none      sine of a in radians (cos &amp; tan)
    asin(a)      12       none      arc sine of a (acos &amp; atan)
</pre>

