
<h2><a name="3">Chapter 3: Conventions</a></h2>

      <UL>
         <li><a href="#3.1">3.1 Command Line Interpreters</a>
         <li><a href="#3.2">3.2 Command Formats</a>
         <li><a href="#3.3">3.3 Program Options</a>
         <li><a href="#3.4">3.4 File Inputs and Outputs</a>
         <li><a href="#3.5">3.5 Input Formats</a>
         <li><a href="#3.6">3.6 Limits and Error Messages</a>
         <li><a href="#3.7">3.7 Manual Entries</a>
      </UL>

Features common to all the |STAT programs are covered.  This information makes
it easier to learn about new |STAT programs, and serves as a reference for
experienced users.

<h3><a name="3.1">3.1 Command Line Interpreters</a></h3>

<p> |STAT
analyses consist of a series of commands, each on a single line, hence the name
<i>command line</i>.  Commands are typed by users into a command line
interpreter, itself a program that runs the commands typed in.  On MSDOS, there
is no special name given to the command line interpreter.  On UNIX, the command
line interpreters are called <i>shells</i>, and there are several of them.
Users are expected to know the conventions of their command line interpreters.
Some of the examples in this handbook and in the manual entries will not work
because of differences in how command lines are formatted.  Minor modifications
to the examples are sometimes needed.  <p> Some command line interpreters
support in-line editing, which is useful when running |STAT analyses because
data analysis is an iterative process in which minor changes in analyses, and
hence commands, are common.  <p>

<b>Special Characters</b> <br> Command line interpreters have special
characters to perform special tasks.  On both MSDOS and UNIX, there are special
characters for file input, output, and pipe redirection:

<pre>
&lt;	redirect standard input from the following file 
&gt;	redirect standard output to the following file
|	redirect standard output to the following command
</pre>

UNIX and MSDOS both have patterns (sometimes called ``wildcards'') to
match file names.  For example, <tt>*.c</tt> matches all files that end with a
<tt>c</tt> suffix.  Also, the <tt>?</tt> can be used in patterns to match any
one character.  An important difference between UNIX and MSDOS command line
interpreters is that on UNIX, the pattern matching is part of the shell, and so
is available to every program, while on MSDOS, it is part of only some
programs.  <p> It is sometimes necessary to quote the special meaning of
special characters so that they are not seen by the command line interpreter.
For example, an expression for <tt>dm</tt> might contain the symbols <tt>*</tt>
for multiplication or <tt>&lt;</tt> for comparison.  Both these characters are
special to UNIX shells, while only <tt>&lt;</tt> is special to MSDOS.  The
blank space and tab characters are special on both UNIX and MSDOS, and are used
to separate command line arguments.  Special characters can be quoted by
enclosing command line arguments in double quotes.  For example, <tt>dm</tt>
expressions may contain special characters, and strings may contain spaces.

<pre>dm  "if x1 &gt; 10 then 'Large number on line:' else SKIP"  INLINE</pre>

<h3><a name="3.2">3.2 Command Formats</a></h3>

<p> |STAT
programs are run on UNIX and MSDOS by typing the name of the program, program
options, and program operands (e.g., expressions or file names).  Program
names, options, and operands, are separated or <i>delimited</i> by blank space.
On UNIX, program names are lower case, while on the case-insensitive MSDOS,
they are always upper case, although users can type the names in lower case.
Program options and operands can be complex, so it is sometimes useful to
insert spaces into an option value or an operand, either to modify the output
or to make the command line more readable.  This is done by quoting (with
double quotes) the parts that should be kept together.  <p>

<b>Simple Commands</b> <br> A simple command consists of a program name,
program options delimited with minus signs, and program operands, such as file
or variable names.  Here are some examples: 

<pre>
dm  x1+x2  x3/x4
calc  model
regress  -p  age  height  weight
desc  -h  -i 1  -m 0  -cfp
series  1  100  .5
probdist  random  normal  100
</pre>

<p>
<b>Pipelines of Commands</b> <br> A pipeline of commands is a series of simple
commands joined by the pipe symbol, <tt>|</tt>.  In a pipeline, the output from
one simple command is the input to the next command in the pipeline.  The
following pipeline creates a series of numbers from 1 to 100, transforms it by
using the <tt>dm</tt> logarithm function, and then makes a histogram of the
result.  

<pre>series 1 100  |  dm logx1  |  desc -h</pre>

The following pipeline
abuts three files beside one another, and passes the result to the
<tt>regress</tt> program, which prints their correlation matrix.  

<pre>abut age height weight  |  regress -r age height weight</pre>

Note that the operands to
<tt>abut</tt> are file names, while those for <tt>regress</tt> are variable
names, which could be different if desired.  If they were always supposed to be
the same, then this constraint could be encoded in a shell script or batch
file.  <p>

<b>Batch Files and Shell Scripts</b> <br> Because the |STAT programs work well
together, and because most data analysis is routine, it is often advantageous
to save a series of commands in a file for later analyses.  Both UNIX and MSDOS
support this, MSDOS with <i>batch</i> files and UNIX with <i>shell</i> scripts.
Batch files and shell scripts also support variables, some set by command line
calls and some set inside the command file.  They provide |STAT with a simple
but effective programming facility.

<h3><a name="3.3">3.3 Program Options</a></h3>

<p> Program options allow the user to
control how a program works by requesting custom or extra analysis.  Without
options, |STAT programs provide the simplest or most common behavior by
default.  Program options conform to the standard UNIX option parsing
convention (Hemenway &amp; Armitage, 1984) by using the <tt>getopt</tt> option
parser.  In this standard, all program options are single characters preceded
by a minus sign.  For example, <tt>-a</tt> and <tt>-X</tt> are both options.
All program options must precede operands (such as file names, variable names,
or expressions).  Some options require values, and these should follow the
option.  For example, the <tt>pair</tt> plotting function allows setting the
height of the plot with the <tt>-h</tt> option: <tt>-h 30</tt> would set the
plot height to 30 lines.  There should be a space between an option and its
value.  Options that do not take values (logical options) can be grouped or
``bundled'' to save typing.  For example, the descriptive statistics program,
<tt>desc</tt>, has options for requesting a histogram, a table of frequencies,
and a table of proportions.  These can be requested with the bundle of options:
<tt>-hfp</tt> instead of the longer: <tt>-h -f -p</tt>.  <p> There are some
special conventions used with the <tt>getopt</tt> option parser.  A double
dash, <tt>--</tt>, by itself signals the end of the options, which can be
useful when the first operand begins with <tt>-</tt> and it would be
misinterpreted as an option.  For programs that take files as operands (e.g.,
<tt>abut</tt>, <tt>calc</tt>), a solitary <tt>-</tt> means to read from the
standard input, which can be useful to insert the output of a pipeline in a set
of files.  For example, the <tt>abut</tt> program can read several files with
the standard input inserted with the following command line.  

<pre>series 1 20 |  abut file1 file2 - file3</pre>

The output would be four columns, the third of
which would be the series 1 to 20.  <p> The same options can usually be
specified more than once on a command line.  For logical options (those that
turn on or off a feature), repetition usually has no effect.  For options that
take values, such as the width of a plot, respecifying an option resets it to a
new value.  Exceptions to these rules for specific options are mentioned in
program manual entries.  

<p>
<b>Table of Option Rules</b>
<pre>
-x        options are single letters preceded by minus
-h 30     option values must follow the option after a space
-nve      logical options can be bundled
--        signals the end of the options
-         insert standard input to operands of file-reading program
</pre>

<p>
<b>Standard Options</b> <br> All |STAT programs using the standard option
parser, <tt>getopt</tt>, have standard options to get information online.  The
information reported by the program is always accurate, while the printed
documentation may not be up to date, or may not apply to the particular version
(e.g., limits on MSDOS may be smaller than on UNIX).  

<pre>
-L  prints a list of program limits
-O  prints a summary of program options
-V  prints version information
</pre>

<h3><a name="3.4">3.4 File Inputs and Outputs</a></h3>

<p>
Most of the |STAT programs are filters.  That means they read from the standard
input and write to the standard output.  By default, the standard input is the
keyboard, and the standard output is the screen.  The standard input and output
can independently be ``redirected'' using the special characters:
<tt>&lt;</tt>, to redirect the standard input from an immediately following
file name, <tt>&gt;</tt>, to redirect the standard output to a file.  Also, the
pipe character <tt>|</tt>, can connect the output from one program to the input
to another.  (Some of these features are not available on early versions of
MSDOS (before version 2.0).) The following command says for the <tt>anova</tt>
program to read from the file <tt>anova.in</tt>.  

<pre>anova  &lt;  anova.in</pre>

The output would go to the screen, by default.  The following command saves the
above output to the file <tt>anova.out</tt>.  

<pre>anova  &lt;  anova.in  &gt; anova.out</pre>

Never do this: 

<pre>anova  &lt;  data  &gt;  data          # Never Do This!</pre>

Never make the input file the same as the output file, or you will
lose the file; the output file is created (and zeroed) by the command line
interpreter before the input file is read.  Temporary files should be used
instead.  Here is an example of output redirection to save 50 random normal
numbers.  

<pre>probdist random normal 50  &gt;  numbers</pre>

In English, this is
read: ``A random sample of 50 numbers is created and saved in the file
<tt>numbers</tt>.  This file of numbers could be used as input to the
descriptive statistics program, <tt>desc</tt>.  The intermediate file,
<tt>numbers</tt>, could be avoided by using a pipeline.  

<pre>probdist random normal 50  |  desc</pre>

To save the result of the above analysis in a file called
<tt>results</tt>, output redirection would be used.  

<pre>probdist random normal 50  |  desc  &gt;  results</pre>

<p> Although pipes are supported on MSDOS,
they are not efficient and they require that there is enough space for
temporary files to hold the contents of the pipes (temporary files with names
like <tt>PIPE%1.$$$</tt>).  This can make input and output redirection without
pipes a better choice for speed, especially in command scripts, called ``batch
files'' on MSDOS.  <p>

<b>Keyboard Input</b> <br> If a program is expecting input from the keyboard
(ie. the standard input has not been redirected from a file or pipe), a prompt
will be printed on the screen.  Often, input from the keyboard is a mistake;
most people do not type directly into an analysis program but prepare a file
with their preferred editor and use that file as input.  

<pre>prompt: desc</pre>

desc: reading input from terminal: user types input, followed by end of file:
^D on UNIX, ^Z on MSDOS In all examples of keyboard input, the sequence
<tt>^X</tt> will be used for control characters like control-x (hold down the
<tt>CTRL</tt> key and type the letter <tt>x</tt>).  On UNIX, end of input from
the keyboard is signaled by typing <tt>^D</tt>.  MSDOS users type <tt>^Z</tt>.

<h3><a name="3.5">3.5 Input Formats</a></h3>

<p> |STAT programs have simple input formats.
Program input is read until the end of file, <i>EOF</i>, is found.  End of file
in disk files is done by the system; no special marking characters are needed
nor allowed.  

<p> Input <i>fields</i> (visibly distinguishable <i>words</i>)
are separated by <i>whitespace</i> (blank spaces, tabs, newlines).  For most
programs, fields in lines with embedded spaces can be enclosed by single or
double quotes.  Most |STAT analysis programs ignore blank input lines used to
improve the human-readability of the data.  However, blank lines are meaningful
to some data manipulation programs, so when there are unexpected results, it is
often instructive to run a file through <tt>validata</tt>.  <p>

<b>Suggestion: Staged Analysis</b> <br> It is usually a good idea to build a
complex command, such as a pipeline, in stages.  At each stage, a quick visual
inspection of the output catches most errors you might make.  <p>

<b>Data Types</b> <br> |STAT programs recognize several types of data: label
and variable names, numbers (integers and real numbers), and some programs can
deal with missing values, denoted by <tt>NA</tt>.  Label and variable names
begin with an alphabetic character (a-z or A-Z), and can be followed by any
number of alphanumerics (a-z, A-Z, 0-9) and underscores.  There are three types
of numbers: integers, real numbers with a decimal point, and numbers in
exponential scientific notation.  Integers are positive or negative numbers
with no decimal point, or if they have a decimal point, they have no non-zero
digits after the decimal point.  Exponential notation numbers are numbers of
the form <tt>xxx.yyyEzz</tt>.  They may have digits before an optional decimal
point or after it, and the number after the <tt>E</tt> or <tt>e</tt> is a power
of ten multiplier.  For example, <tt>1.2e-6</tt> is 1.2 times the inverse of
one million.  <p>

<b>Caveat: Appearances Can Be Deceiving</b> <br> Inputs that look like they
line up might not appear so to |STAT programs.  For example, the following data
might appear to have four columns, but have a variable number.  Also, the
columns that look like they line up to a person, do not line up to |STAT
programs.  

<pre>
a   b   c   d
e       f   g
h   i       j
</pre>

Here is how |STAT
programs interpret this input: 

<pre>
a   b   c   d
e   f   g
h   i   j
</pre>

This
difference could be found with the <tt>validata</tt> utility program, which
would report for both formats above: 

<pre class=output>
validata: Variable number of columns at line 2
Col   N  NA alnum alpha   int float other  type   min   max
  1   3   0     3     3     0     0     0 alnum     0     0
  2   3   0     3     3     0     0     0 alnum     0     0
  3   3   0     3     3     0     0     0 alnum     0     0
  4   1   0     1     1     0     0     0 alnum     0     0
</pre>

<h3><a name="3.6">3.6 Limits and Error Messages</a></h3>

<p> There is a system-dependent limit on the count of
characters in an input line: on small systems, 512 characters, and on large
ones, 1024.  Many programs use dynamic memory allocation so the memory
available on a machine will determine the size of data sets that can be
analyzed.  Integer overflow is not checked, so numbers like data counts are
limited on 16 bit machines to 32767; in practice, this has not presented
problems.  All calculations are done with double precision floating point
numbers, but overflow (exceeding the maximum allowed double precision number,
about 10 to the 38th power) and underflow (loss of precision of a tiny non-zero
result being rounded to 0.0) are not checked.  Program specific limits can be
found in most programs with the <tt>-L</tt> option.  The programs are not
robust when used on highly variable data (differences of several orders of
magnitude), very large numbers, or large datasets (more than 10,000 values).
<p> All error and warning messages (1) identify the program detecting the
problem (useful when pipelines or command scripts are used), (2) print
diagnostic information, (3) sound a bell, and for errors, (4) cause an exit.
All error and warning messages are printed on the diagnostic output (that is
<tt>stderr</tt> for C lovers), so they will be seen even if the standard output
is redirected to a file.  All |STAT programs exit with a non-zero exit status
on error and a zero exit status after a successful run.  <p>

<b>Common Error Messages</b> <br> Some errors and messages are common to
several programs.  They are explained below.  Other messages should be self-
explanatory.  

<pre>
Not enough (or no) input data
    There were no data points read, or not enough to make sense
Too many xxxx's; at most N allowed
    Too many of something were in the input (e.g., columns or variables)
Cannot open 'file'
    The named file could not be opened for reading
No storage space left for xxxx
    The program has run out of dynamic memory for internal storage
'string' (description) is not a number
    The described object whose input value was 'string' was non-numerical
N operand(s) ignored on command line
    Operands (e.g., files) on the command line are ignored by this program
VALUE is an illegal value for the TYPE
    The provided value was out of the legal range for the given type
Ragged input file
    The program expects a uniform number of input columns
</pre>

<h3><a name="3.7">3.7 Manual Entries</a></h3>

<p> |STAT manual entries contain detailed
information about each of the programs.  They describe the effects of all the
options.

<p>

<b>On-Line Manuals</b> <br> On UNIX systems, the manual entries for |STAT
programs are available online with the <tt>manstat</tt> program.  UNIX system
administrators might prefer to install the |STAT manuals in a public place, so
they might be available with the standard UNIX <tt>man</tt> program.  On MSDOS
systems, manual entries might be available online with a batch file that types
pre-formatted manuals.  The following will print the online manual for the
<tt>anova</tt> program.  

<pre>manstat anova</pre>

Most programs print a summary of
their options with the <tt>-O</tt> option.  The following will print a summary
of the options available with the <tt>desc</tt> descriptive statistics program.

<pre>desc -O</pre>

<p>
<b>Manual Entries on the Web</b>
<br>
Manual entries are available on the Web:

<a href="http://www.acm.org/~perlman/stat/doc/"
	>Web-based Manual Entries</a>

<p>

<b>UNIX Manual Conventions</b> <br> UNIX manual entries are often considered
cryptic, especially for new users.  It helps to know the conventions used in
writing manual entries.  In the following table, the contents of the different
manual entry sections are summarized.  

<dl>
<dt>ALGORITHMS<dd>
    sources or descriptions of algorithms
<dt>BUGS<dd>
    limitations or known deficiencies in the program
<dt>DESCRIPTION<dd>
    details about the workings of the program,
    and information about operands
<dt>EXAMPLES<dd>
    examples of command lines showing expected use of the program
<dt>FILES<dd>
    files used by the program (e.g., temporary files)
<dt>LIMITS<dd>
    limits built into the program should be determined with the -L option
<dt>NAME<dd>
    the name and purpose of the program
<dt>OPTIONS<dd>
    detailed information about command line options (see the -O option)
<dt>SYNOPSIS<dd>
    a short summary of the option/operand syntax for the program
    (items enclosed in square brackets are optional)
</dl>

