
<h3><a name="dm">DM: Tutorial and Manual</a></h3>
<!-- This Table of Contents generated by tochtml -->

         <UL>
            <li><a href="#intro">Introduction</a>
            <UL>
               <li><a href="#examples">Introductory Examples</a>
               <li><a href="#exprs">Simple Expressions</a>
               <li><a href="#condex">Line Extraction Based on Conditions</a>
            </UL>
            <li><a href="#dt">Data Types</a>
            <UL>
               <li><a href="#string">String Data</a>
               <li><a href="#numerical">Numerical Data</a>
               <li><a href="#special">Special Variables</a>
            </UL>
            <li><a href="#ui">User Interface</a>
            <UL>
               <li><a href="#uiexpr">Expressions</a>
               <li><a href="#argmode">Argument Mode</a>
               <li><a href="#filemode">Expression File Mode</a>
               <li><a href="#intermode">Interactive Mode</a>
               <li><a href="#input">Input</a>
               <li><a href="#output">Output</a>
            </UL>
            <li><a href="#operations">Operations</a>
            <UL>
               <li><a href="#stringop">String Operations</a>
               <li><a href="#numops">Numerical Operators</a>
               <li><a href="#logops">Logical Operators</a>
               <li><a href="#condexp2">Conditional Expressions</a>
            </UL>
            <li><a href="#exprsyntax">Expression Syntax</a>
            <UL>
               <li><a href="#unary">Unary Operators</a>
               <li><a href="#binary">Binary Operators</a>
            </UL>
            <li><a href="#examples2">Some Examples</a>
         </UL>

<h4><a name="intro">Introduction</a></h4>

<p> <tt>dm</tt> is a data manipulating program with many
operators for manipulating columnated files of numbers and strings.
<tt>dm</tt> helps avoid writing little BASIC or C programs every time some
transformation to a file of data is wanted.  To use <tt>dm</tt>, a list of
expressions is entered, and for each line of data, <tt>dm</tt> prints the
result of evaluating each expression. 

<h5><a name="examples">Introductory Examples</a></h5>
Usually, the input to <tt>dm</tt> is a file of lines, each with the same number
of fields.  Put another way, <tt>dm</tt>'s input is a file with some set number
of columns.  <p> <i>Column Extraction:</i> <tt>dm</tt> can be used to extract
columns.  If <tt>data</tt> is the name of a file of five columns, then the
following will extract the 3rd string followed by the 1st, followed by the 4th,
and print them to the standard output.  

<pre>dm  s3  s1  s4  &lt;  data</pre>

Thus
<tt>dm</tt> is useful for putting data in a correct format for input to many
programs, notably the |STAT data analysis programs.  <i>Warning:</i> If a
column is missing (e.g., you access column 3 and there is no third column in
the input), then the value of the access will be taken from the previous input
line.  This <i>feature</i> must be considered if there are blank lines in the
input; it may be best to remove blank lines, with <tt>dm</tt> or some other
filter program.  

<h5><a name="exprs">Simple Expressions</a></h5>
In the preceding example,
columns were accessed by typing the letter <tt>s</tt> (for string) followed by
a column number.  The numerical value of a column can be accessed by typing
<tt>x</tt> followed by a column number.  This is useful to form simple
expressions based on columns.  Suppose <tt>data</tt> is a file of four
numerical columns, and that the task is to print the sum of the first two
columns followed by the difference of the second two.  The easiest way to do
this is with: 

<pre>dm  x1+x2  x3-x4  &lt;  data</pre>

Almost all arithmetic operations
are available and expressions can be of arbitrary complexity.  Care must be
taken because many of the symbols used by <tt>dm</tt> (such as <tt>*</tt> for
multiplication) have special meaning when used in UNIX (though not MSDOS).
Problems can be avoided by putting expressions in quotes.  For example, the
following will print the sum of the squares of the first two columns followed
by the square of the third, a simple Pythagorean program.  

<pre>dm "x1*x1+x2*x2"  'x3*x3'  &lt;  data</pre>

<h5><a name="condex">Line Extraction Based on Conditions</a></h5>
<tt>dm</tt> allows printing values that depend on conditions.  The <tt>dm</tt>
call 

<pre>dm  "if x1 &gt;= 100 then INPUT else NEXT"  &lt;  data</pre>

will print only
those lines that have first columns with values greater than or equal to 100.
The variable <tt>INPUT</tt> refers to the whole input line.  The special
variable <tt>NEXT</tt> instructs <tt>dm</tt> to stop processing on the current
line and go to the next.  

<h4><a name="dt">Data Types</a></h4>
<h5><a name="string">String Data</a></h5>
To access or print a column in a
file, the string variable, <tt>s</tt>, is provided.  <tt>s</tt>i (the letter
<tt>s</tt> followed by a column number, such as <tt>5</tt>) refers to the ith
column of the input, treated as a string.  The most simple example is to use an
<tt>s</tt>i as the only part of an expression.  

<pre>dm  s2  s3  s1</pre>

will print
the second, third and first columns of the input.  One special string is called
<tt>INPUT</tt>, and is the current input line of data.  String constants in
expressions are delimited by single or double quotes.  For example: 

<pre>"I am a string"</pre>

<h5><a name="numerical">Numerical Data</a></h5>
Constant numbers like <tt>123</tt> or
<tt>14.6</tt> can be used alone or with other expressions.  Two general
numerical variables are available To refer to the input columns, there is
<tt>x</tt>i and for the result of evaluated expressions, there is <tt>y</tt>i.
<tt>x</tt>i refers to the ith column of the input, treated as a number.
<tt>x</tt>i is the result of converting <tt>s</tt>i to a number.  If
<tt>s</tt>i contains non-numerical characters, <tt>x</tt>i may have strange
values.  A common use of the <tt>x</tt>i is in algebraic expressions.  

<pre>dm x1+x2  x1/x2</pre>

will print out two columns, first the sum of the first two input
columns, then their ratio.  

<p> The value of a previously evaluated expression
can be accessed to avoid evaluating the same sub-expression more than once.
<tt>y</tt>i refers to the numerical value of the ith expression.  Instead of
writing: 

<pre>dm  x1+x2+x3  (x1+x2+x3)/3</pre>

the following would be more efficient: 

<pre>dm  x1+x2+x3     y1/3</pre>

<tt>y1</tt> is the value of the first
expression, <tt>x1+x2+x3</tt>.  String values of expressions are unfortunately
inaccessible.  

<p> Indexing numerical variables is usually done by putting the
index after <tt>x</tt> or <tt>y</tt>, but if value of the index is to depend on
the input, such as when there are a variable number of columns, and only the
last column is of interest, the index value will depend on the number of
columns.  If a computed index is desired for <tt>x</tt> or <tt>y</tt> the index
should be an expression in square brackets following <tt>x</tt> or <tt>y</tt>.
For example, <tt>x[N]</tt> is the value of the last column of the input.
<tt>N</tt> is a special variable equal to the number of columns in
<tt>INPUT</tt>.  There is the option to use <tt>x1</tt> or <tt>x[1]</tt> but
<tt>x1</tt> will execute faster so computed indexes should not be used unless
necessary. 

<h5><a name="special">Special Variables</a></h5>

<tt>dm</tt> offers some special
variables and control primitives for commonly desired operations.  Many of the
special variables have more than one name to allow more readable expressions.
Many can be abbreviated, and the short forms will be shown in square brackets.

<pre>
N         the number of columns in the current input line
SUM       the sum of the numbers on the input line
INLINE    the line number of the input (initially 1.0)
OUTLINE   the number of lines so far output (initially 0.0)
RAND [R]  a random number uniform in [0,1) (may be followed by a seed)
INPUT [I] the original input line, all spaces, etc. included
NIL       the empty expression (often used with a test)
KILL [K]  stop processing the current line and produce no output
NEXT      synonym for KILL
SKIP      synonym for KILL
EXIT [E]  exit immediately (useful after a search)
</pre>

<h4><a name="ui">User Interface</a></h4>
<h5><a name="uiexpr">Expressions</a></h5>
Expressions are written in
common computer language syntax, and can have spaces or underscores inserted
for readability anywhere except (1) in the middle of constants, and (2) in the
middle of multicharacter operators such as <tt><=</tt> (less than or equal to).
Four modes are available for specifying expressions to <tt>dm</tt>.  They
provide the choice of entering expressions from the terminal or a file, and the
option to use <tt>dm</tt> interactively or in batch mode.  

<h5><a name="argmode">Argument Mode</a></h5>

The most common but restrictive mode is to supply expressions as
arguments on the command line call to <tt>dm</tt>, as featured in previous
examples.  The main problem with this mode is that many special characters in
UNIX and MSDOS are used as operators, requiring that many expressions be
quoted.  The main advantage is that this mode is most useful in constructing
pipelines and shell scripts.  

<h5><a name="filemode">Expression File Mode</a></h5>
Another non-interactive method is to supply <tt>dm</tt> with a file with expressions in it
(one to each line) by calling <tt>dm</tt> with: 

<pre>dm  Efilename</pre>

where
<tt>filename</tt> is a file of expressions.  This mode makes it easier to use
<tt>dm</tt> with pipelines and redirection.  

<h5><a name="intermode">Interactive Mode</a></h5>
<tt>dm</tt> can also be used interactively by calling <tt>dm</tt> with no
arguments.  In interactive mode, <tt>dm</tt> will first ask for a file of
expressions.  If the expressions are not in a file, type <tt>RETURN</tt> when
asked for the expression file, and they can be entered interactively.  A null
filename tells <tt>dm</tt> to read expressions from the terminal.  In terminal
mode, <tt>dm</tt> will prompt with the expression number, and print out how it
interprets what is typed in if it has correct syntax, otherwise it allows
corrections.  When the last expression has been entered, an empty line informs
<tt>dm</tt> there are no more.  If the expressions are in a file, type in the
name of the file, and <tt>dm</tt> will read them from there. 

<h5><a name="input">Input</a></h5>
If <tt>dm</tt> is used in interactive mode, it will prompt for an input file.
A file name can be supplied or a <tt>RETURN</tt> without a file name tells
<tt>dm</tt> to read data from the terminal.  Out of interactive mode,
<tt>dm</tt> will read from the standard input.  

<p> <tt>dm</tt> reads data a
line at a time and stores that line in a string variable called <tt>INPUT</tt>.
<tt>dm</tt> then takes each column in <tt>INPUT</tt>, separated by spaces or
tabs, and stores each in the string variables, <tt>s</tt>i.  <tt>dm</tt> then
tries to convert these strings to numbers and stores the result in the number
variables, <tt>x</tt>i.  If a column is not a number (e.g., it is a string)
then its numerical value will be inaccessible, and trying to refer to such a
column will cause an error message.  The number of columns in a line is stored
in a special variable called <tt>N</tt>, so variable numbers of columns can be
dealt with gracefully.  The general control structure of <tt>dm</tt> is
summarized in the following display.  

<pre>
read in n expressions; e1, e2, ..., en.
repeat while there is some input left
     INPUT  = <next line from input file>
     N      = <number of fields in INPUT>
     SUM    = 0
     RAND   = <a new random number in [0,1)>
     INLINE = INLINE + 1
     for i  = 1 until N do
          si  = <ith string in INPUT>
          xi  = <si converted to a number>
          SUM = SUM + xi
     for i = 1 until n do
          switch on <value of ei>
               case EXIT: <stop the program>
               case KILL: <go to get new INPUT>
               case NIL : <go to next expression>
               default  :
                    OUTLINE = OUTLINE + 1
                    yi = <value of ei>
                    if (ei not X'd) print yi
     <print a newline character>
</pre>

<h5><a name="output">Output</a></h5>
In interactive
mode, <tt>dm</tt> will ask for an output file (or pipe, on UNIX only).  

<pre>
Output file or pipe:
</pre>

A filename, a ``pipe command,'' or just <tt>RETURN</tt>
can be entered.  A null filename tells <tt>dm</tt> to print to the terminal.
If output is being directed to a file, the output file should be different from
the input file.  <tt>dm</tt> will ask permission to overwrite any file that
contains anything, but that does not mean it makes sense to write the file it
is reading from.  

<p> On UNIX, the output from <tt>dm</tt> can be redirected to
another program by having the first character of the output specification be a
pipe symbol, the vertical bar: <tt>|</tt>.  For example, the following line
tells <tt>dm</tt> to pipe its output to <tt>tee</tt> which prints a copy of its
output to the terminal, and a copy to the named file.  

<pre>Output file or pipe: | tee dm.save</pre>

<p> Out of interactive mode, <tt>dm</tt> prints to the
standard output.  

<p> <tt>dm</tt> prints the values of all its expressions in
<tt>%.6g</tt> format for numbers (maintaining at most six digits of precision
and printing in the fewest possible characters), and <tt>%s</tt> format for
strings.  A tab is printed after every column to insure separation. 

<h4><a name="operations">Operations</a></h4>

<tt>dm</tt> offers many numerical, logical, and string
operators.  The operators are evaluated in the usual order (e.g., times before
plus) and expressions tend be evaluated from left to right.  Parentheses can be
used to make the order of operations clear.  The way <tt>dm</tt> interprets
expressions can be verified by entering them interactively on UNIX, in which
case <tt>dm</tt> prints a fully parenthesized form.  

<p> An assignment operator
is not directly available.  Instead, variables can be evaluated but not printed
by using the expression suppression flag, <tt>X</tt>.  If the first character
of an expression is <tt>X</tt>, it will be evaluated, but not printed.  The
value of a suppressed expression can later be accessed with the expression
value variable, <tt>y</tt>i.  

<h5><a name="stringop">String Operations</a></h5>
Strings can be
lexically compared with several comparators: <tt>&lt;</tt> or <tt>LT</tt> (less-
than), <tt><=</tt> or <tt>LE</tt> (less-than or equal), <tt>=</tt> or
<tt>EQ</tt> (equal), <tt>!=</tt> or <tt>NE</tt> (not equal), <tt>&gt;=</tt> or
<tt>GE</tt> greater-than or equal), and <tt>&gt;</tt> or <tt>GT</tt> (greater
than).  They return <tt>1.0</tt> if their condition holds, and <tt>0.0</tt>
otherwise.  For example, 

<pre>"abcde"  <=  'eeek!'</pre>

is equal to <tt>1.0.</tt>
The length of strings can be found with the <tt>len</tt> operator.  

<pre>len 'five'</pre>

evaluates to 4, the length of the string argument.  The character
<tt>#</tt> is a synonym for the <tt>len</tt> operator.  The numerical type of a
string can be checked with the <tt>number</tt> function, which returns 0 for
non-numerical strings, 1 for integer strings, and 2 for real numbers
(scientific notation or strings with non-zero digits after the decimal point).

<p> Individual characters inside strings can be accessed by following a string
with an index in square brackets.  

<pre>"abcdefg"[4]</pre>

is the ASCII character
number (164.0) of the 4th character in <tt>abcdefg</tt>.  Indexing a string is
mainly useful for comparing characters because it is not the character that is
printed, but the character number.  A warning is appropriate here: 

<pre>s1[1] =  '*'</pre>

will result in an error because the left side of the <tt>=</tt> is a
number, and the right hand side is a string.  The correct (although inelegant)
form is: 

<pre>s1[1]  =  '*'[1]</pre>

<p> A substring test is available. The
expression: 

<pre>string1  C  string2</pre>

will return <tt>1.0</tt> if
<tt>string1</tt> is somewhere in <tt>string2</tt>.  This can be used as a test
for character membership if string1 has only one character.  Also available is
<tt>!C</tt> which returns <tt>1.0</tt> if <tt>string1</tt> is NOT in
<tt>string2</tt>.  

<h5><a name="numops">Numerical Operators</a></h5>
The numerical comparators
are: 

<pre>
&lt;  &lt;=  =  !=  &gt;=  &gt;
LT LE  EQ NE  GE GT
</pre>

and have the analogous
meanings as their string counterparts.  

<p> The binary operators, <tt>+</tt>
(addition), <tt>-</tt> (subtraction or "change-sign"), <tt>*</tt>
(multiplication), and <tt>/</tt> (division) are available.  Multiplication and
division are evaluated before addition and subtraction, and are all evaluated
left to right.  Exponentiation, <tt>^</tt>, is the binary operator of highest
precedence and is evaluated right to left.  Modulo division, <tt>%</tt>, has
the same properties as division, and is useful for tests of even/odd and the
like.  NOTE: Modulo division truncates its operands to integers before
dividing.  

<p> Several unary functions are available: <tt>l</tt> (natural log
<tt>[log]</tt>), <tt>L</tt> (base ten log <tt>[Log]</tt>), <tt>e</tt>
(exponential <tt>[exp]</tt>), <tt>a</tt> (absolute value <tt>[abs]</tt>),
<tt>f</tt> (floor <tt>[floor]</tt>), <tt>c</tt> (ceiling <tt>[ceil]</tt>).
Their meaning can be verified in the UNIX Programmer's Manual.  Single letter
names for these functions or the more mnemonic strings bracketed after their
names can be used.  Also available are trigonometric functions that work on
degrees in radians: <tt>sin cos tan asin acos atan</tt>.

<h5><a name="logops">Logical Operators</a></h5>
Logical operators are of lower precedence than any other
operators.  Both logical AND, <tt>&</tt> and OR <tt>|</tt> can be used to form
complicated tests.  For example, to see if the first three columns are in
either increasing or decreasing order, one could test if <tt>x2</tt> was
between <tt>x1</tt> and <tt>x3</tt>: 

<pre>x1<x2  &amp;  x2<x3  |  x1>x2  &amp;  x2>x3</pre>

would equal <tt>1.0</tt> if the condition was satisfied.  Parentheses are
unnecessary because <tt>&lt;</tt> and <tt>&gt;</tt> are of higher precedence than
<tt>&</tt> which is of higher precedence than <tt>|</tt>.  The above expression
could be written as: 

<pre>x1 LT x2  AND  x2 LT x3  OR  x1 GT x2  AND  x2 GT x3</pre>

by using synonyms for the special character operators.  This is useful to avoid
the special meaning of characters in command lines.  The unary logical
operator, <tt>!</tt> (NOT), evaluates to 1.0 if its operand is <tt>0.0</tt>,
otherwise it equals <tt>0.0</tt>.  Many binary operators can be immediately
preceded by <tt>!</tt> to negate their value.  <tt>!=</tt> is "not equal to,"
<tt>!|</tt> is "neither," <tt>!&</tt> is "not both," and <tt>!C</tt> is "not
in."

<h5><a name="condexp2">Conditional Expressions</a></h5>
The expressions: 

<pre>if expression1 then expression2 else expression3
   expression1  ?   expression2   :  expression3</pre>
   
evaluate to
<tt>expression2</tt> if <tt>expression1</tt> is non-zero, otherwise they
evaluate to <tt>expression3</tt>.  The first form is more mnemonic than the
second which is consistent with C syntax.  Upper case names can be used in
their stead.  Both forms have the same meaning.  <tt>expression1</tt> has to be
numerical, <tt>expression2</tt> or <tt>expression3</tt> can be numerical or
string.  For example, The following expression will filter out lines with the
word <tt>bad</tt> in them.  

<pre>if  'bad'  C  INPUT  then  KILL  else  INPUT</pre>

As another example, the following expression will print the ratio of columns
two and three if (a) there are at least three columns, and (b) column three is
not zero.  

<pre>if  (N &gt;= 3)  &amp;  (x3 != 0)  then  x2/x3  else  'bad line'</pre>

These are the only expressions, besides <tt>s</tt>i or a string constant that
can evaluate to a string.  If a conditional expression does evaluate to a
string, then it CANNOT be used in some other expression.  The conditional
expression is of lowest precedence and groups left to right, however
parentheses are recommended to make the semantics obvious. 

<h4><a name="exprsyntax">Expression Syntax</a></h4>
Arithmetic expressions may be formed using
variables (with <tt>x</tt>i and <tt>y</tt>i) and constants and can be of
arbitrary complexity.  In the following table, unary and binary operators are
listed along with their precedences and a brief description.  All unary
operators are prefix except string indexing, <tt>[]</tt>, which is postfix.
All binary operators are infix.  

<p> Operators of higher precedence are
executed first.  All binary operators are left associative except
exponentiation, which groups to the right.  An operator, <tt>O</tt>, is left
associative if <tt>xOxOx</tt> is parsed as <tt>(xOx)Ox</tt>, while one that is
right associative is parsed as <tt>xO(xOx)</tt>.  

<h5><a name="unary">Unary Operators</a></h5>
<pre>
  op    prec   description
  sin     10   sine of argument degrees in radians
  cos     10   cosine of argument degrees in radians
  tan     10   tangent of argument degrees in radians
  asin    10   arc (inverse) sine function
  acos    10   arc (inverse) cosine function
  atan    10   arc (inverse) tangent function
  sqrt    10   square root function
  log     10   base e logarithm [l]
  Log     10   base 10 logarithm [L]
  exp     10   exponential [e]
  abs     10   absolute value [a]
  ceil    10   ceiling (rounds up to next integer) [c]
  floor   10   floor (rounds down to last integer) [f]
  len     10   number of characters in string [#]
  number  10   report if string is a number (0 non, 1 int, 2 real)
  []      10   ASCII number of indexed string character
  -        9   change sign
  !        4   logical not (also NOT, not)
</pre>

<h5><a name="binary">Binary Operators</a></h5>
<pre>
  op    prec   description
  ^        8   exponentiation
  *        7   multiplication
  /        7   division
  %        7   modulo division
  +        6   addition
  -        6   subtraction
  =        5   test for equality (also EQ; opposite !=, NE)
  &gt;        5   test for greater-than (also GT; opposite &lt;=, LE)
  &lt;        5   test for less-than (also LT; opposite, &gt;=, GE)
  C        5   substring (opposite !C)
  &amp;        4   logical AND (also AND, and; opposite !&amp;)
  |        3   logical OR (also OR, or; opposite !|)
</pre>

<p>

<h4><a name="examples2">Some Examples</a></h4>
To print lines 10-20 from an input file
<tt>dm.dat</tt>, you could run the following command (note that <tt>x</tt> is
the same as <tt>x0</tt>, which is the same as <tt>INLINE</tt>, the input line
number).  

<pre>dm  "if x &gt;= 10 and x <= 20 then INPUT else SKIP"  &lt; dm.dat</pre>

<p>
To print all the lines longer than 100 characters, you could run the following:

<pre>dm  "if len(INPUT) &gt; 100 then INPUT else SKIP"  &lt; dm.dat</pre>

<p> To print the
running sums of values in a column, you need to use the <tt>y</tt> variables.
The following will print the running sum of values in the first column.  

<pre>
dm y1+x1
</pre>

To print the running sum of the data in the 5th column is a bit
unintuitive.  <tt>y1</tt> is the value from the previous line of the first
expression, and <tt>x5</tt> is the value of the fifth column on the current
line.  To get the running sum of column 5, you would type: 

<pre>dm y1+x5</pre>

If
the running sum is to come out in the third column, then you would run: 

<pre>
dm &lt;something&gt; &lt;something&gt; y3+x5</pre>

<p> <tt>dm</tt> is good at making tables of
computed values.  In the following example, the <tt>echo</tt> command prints
headings for the columns, and <tt>colex</tt> reformats the output of
<tt>dm</tt>.  <tt>colex</tt> sets the default format to 10.3n (numbers 10 wide,
with 3 decimal places), and prints column 1 in 2i format (2-wide integer) and
column 6 in 6i format (6-wide integer).  The <tt>-t</tt> option to
<tt>colex</tt> stops the printing of tabs after columns.  

<pre>
echo " x       1/x  x**2   sqrt(x)    log(x)"
series 1 10 | dm x1 1/x1 "x1*x1" "sqrt(x1)" "log(x1)" |
    colex -t -F 10.3n 2i1 2 6i3 4-5
</pre>

<pre class=output>
 x       1/x  x**2   sqrt(x)    log(x)
 1     1.000     1     1.000     0.000
 2     0.500     4     1.414     0.693
 3     0.333     9     1.732     1.099
 4     0.250    16     2.000     1.386
 5     0.200    25     2.236     1.609
 6     0.167    36     2.449     1.792
 7     0.143    49     2.646     1.946
 8     0.125    64     2.828     2.079
 9     0.111    81     3.000     2.197
10     0.100   100     3.162     2.303
</pre>

