<h2><a name="1">Chapter 1: Introduction</a></h2>

      <UL>
         <li><a href="#1.1">1.1 Capabilities and Requirements</a>
         <li><a href="#1.2">1.2 Design Philosophy</a>
         <li><a href="#1.3">1.3 Table of |STAT Programs</a>
         <li><a href="#1.4">1.4 Table of UNIX and MSDOS Utilities</a>
         <li><a href="#1.5">1.5 Manual Entries</a>
      </UL>

<p>
The purpose, environment, and philosophy
of the |STAT programs are introduced.

<h3><a name="1.1">1.1 Capabilities and Requirements</a></h3>
<p>
|STAT is a small statistical package I have developed on the UNIX
operating system
(Ritchie &amp; Thompson, 1974)
at the University of California San Diego and at the Wang Institute of
Graduate Studies.
Over twenty programs allow the manipulation and
analysis of data and are complemented by this documentation
and manual entries for each program.
The package has been distributed to hundreds of UNIX sites
and the portability of the package, written in C
(Kernighan &amp; Ritchie, 1979),
was demonstrated when it was ported from UNIX to MSDOS
at Cornell University
on an IBM PC using the
Lattice C compiler.
This handbook is designed to be a tutorial introduction
and reference for the most popular parts of release 5.3 of |STAT
(January, 1987) and updates through February, 1987.
Full reference information on the programs is found in the online
manual entries and in the online options help available with most of the
programs.
<p>

<b>Dataset Sizes</b>
<br>
|STAT programs have mostly been run on small datasets,
the kind obtained in controlled psychological experiments,
not the large sets obtained in surveys or physical experiments.
The programs' performances on datasets with more than about 10,000
points is not known, and the programs should not be used for them.
<p>

<b>System Requirements</b>
<br>
The programs run on almost any version of UNIX.
They are compatible with UNIX systems dating back to Version 6 UNIX
(circa 1975).
On MSDOS, the programs run on versions 2.X through 3.X.
MSDOS versions earlier than 2.0 may not support the pipes often used
with |STAT programs,
and MSDOS version 4.0 formats are not compatible.
Space requirements for MSDOS are about 1 megabyte of disk space,
and at least 96 kilobytes of main memory.
Hard disk storage is preferred, but not mandatory.


<h3><a name="1.2">1.2 Design Philosophy</a></h3>

<p> |STAT programs promote a particular style
of data analysis.  The package is interactive and programmable.  Data analysis
is typically not a single action but an iterative process in which a goal of
understanding some data is approached.  Many tools are used to provide several
analyses of data, and based on the feedback provided by one analysis, new
analyses are suggested.  <p> The design philosophy of |STAT is easy to
summarize.  |STAT consists of several separate programs that can be used apart
or together.  The programs are called and combined at the command level, and
common analyses can be saved in files using UNIX shell scripts or MSDOS batch
files.  <p> Understanding the design philosophy behind |STAT programs makes it
easier to use them.  |STAT programs are designed to be tools, used with each
other, and with standard UNIX and MSDOS tools.  This is possible because the
programs make few assumptions about file formats used by other programs.  Most
of the programs read their inputs from the standard input (what is typed at the
keyboard, unless redirected from a file), and all write to the standard output
(what appears on the screen, unless saved to a file or sent to another
program).  The data formats are readable by people, with fields (columns) on
lines separated by white space (blank spaces or tabs).  Data are line-oriented,
so they can be operated on by many programs.  An example of a filter program on
UNIX and MSDOS that can be used with the |STAT programs is the <tt>sort</tt>
utility, which puts lines in numerical or alphabetical order.  The following
command sorts the lines in the file <tt>input</tt> and saves the result in the
file <tt>sorted</tt>. 

<pre>sort  &lt;  input  &gt;  sorted</pre>

The <tt>&lt;</tt>
symbol causes <tt>sort</tt> to read from <tt>input</tt> and the <tt>&gt;</tt>
causes <tt>sort</tt> to write to the file <tt>sorted</tt>.  Because
<tt>sort</tt> exists on UNIX and MSDOS, it is not necessary to duplicate its
function in |STAT, which does not duplicate existing tools.  (In all following
examples, <tt>this font will be used</tt> to show text (e.g., commands and
program names) that would be seen by people using the programs.  <p> User
efficiency is supported over program efficiency.  That does not mean the
programs are slow, but ease-of-use is not sacrificed to save computer time.
Input formats are simple and readable by people.  There is extensive checking
to protect against invalid analyses.  Output formats of analysis programs are
designed to be easy to understand.  Data manipulation programs are designed to
produce uncluttered output that is ready for input to other programs.  <p> On
UNIX and MSDOS, a <i>filter</i> is a program that reads from the standard
input, also called <tt>stdin</tt> (the keyboard, unless redirected from a file)
and writes to the standard output, also called <tt>stdout</tt> (the screen,
unless redirected to a file).  Most |STAT programs are filters.  They are small
programs that can be used alone, or with other programs.  |STAT users typically
keep their data in a <i>master data file</i>.  With data manipulation programs,
extractions from the master data file are transformed into a format suitable
for input to an analysis program.  The original data do not change, but copies
are made for transformations and analysis.  Thus, an analysis consists of an
extraction of data, optional transformations, and some analysis.  Pictorially,
this can be shown as: 

<pre>data | extract | transform | format | analysis | results</pre>

 where a copy a subset of the data has been extracted, transformed,
reformatted, and analyzed by chaining several programs.  Data manipulation
functions, sometimes built into analysis programs in other packages, are
distinct programs in |STAT.  The use of pipelines, signaled with the pipe
symbol, <tt>|</tt>, is the reason for the name |STAT.

<h3><a name="1.3">1.3 Table of |STAT Programs</a></h3>
<p> |STAT programs are divided into two categories.  There
are programs for data manipulation: data generation, transformation,
formatting, extraction, and validation.  And there are programs for data
analysis: summary statistics, inferential statistics, and data plots.  The data
manipulation programs can be used for tasks outside of statistics.

<p>

<b>Data Manipulation Programs</b>
<pre>
  abut      join data files beside each other
  colex     column extraction/formatting
  dm        conditional data extraction/transformation
  dsort     multiple key data sorting filter
  linex     line extraction
  maketrix  create matrix format file from free-format input
  perm      permute line order randomly, numerically, alphabetically
  probdist  probability distribution functions
  ranksort  convert data to ranks
  repeat    repeat strings or lines in files
  reverse   reverse lines, columns, or characters
  series    generate an additive series of numbers
  transpose transpose matrix format input
  validata  verify data file consistency
</pre>

<b>Data Analysis Programs</b>
<pre>
  anova     multi-factor analysis of variance
  calc      interactive algebraic modeling calculator
  contab    contingency tables and chi-square
  desc      descriptions, histograms, frequency tables
  dprime    signal detection d' and beta calculations
  features  display features of items
  oneway    one-way anova/t-test with error-bar plots
  pair      paired data statistics, regression, scatterplots
  rankind   rank order analysis for independent conditions
  rankrel   rank order analysis for related conditions
  regress   multiple linear regression and correlation
  stats     simple summary statistics
  ts        time series analysis and plots
</pre>

<h3><a name="1.4">1.4 Table of UNIX and MSDOS Utilities</a></h3>

<p> The UNIX and MSDOS
environments are similar, at least as far as |STAT is concerned, but many
command names differ.  The following table shows the pairing of UNIX names with
their MSDOS equivalents.

<pre>
  UNIX           MSDOS          Purpose
  cat            type           print files to stdout
  cd,pwd         cd             change/print working directory
  cp             copy           copy files
  diff           comp           compare and list file differences
  echo           echo           print text to standard output
  grep           find           search for pattern in files
  ls             dir            list files in directory
  mkdir          mkdir          create a new directory
  more           more           paginate text on screen
  mv             rename         move/rename files
  print          print          print files on printer
  rm             del,erase      remove/delete files
  rmdir          rmdir          remove an empty directory
  sort           sort           sort lines in files

  shell-script   batch-file     programming language
  $1,$2          %1,%2          variables
  /dev/tty       con            terminal keyboard/screen
  /dev/null      nul            empty file, infinite sink
</pre>

<h3><a name="1.5">1.5 Manual Entries</a></h3>

|STAT manuals follows the format used on UNIX systems,
and to be honest, it takes some getting used to.  One possible source of
confusion for users is the format of examples in the entries.  The examples are
chosen to work on UNIX using my preferred command shell, <tt>ksh</tt>, so some
translation may be needed for UNIX <tt>csh</tt> users, and for MSDOS users.

See
<a href="http://www.acm.org/~perlman/stat/conventions.html#3.7">Chapter 3</a>
on conventions used in the entries.  Besides the manual entries,
there is online help with most programs with the <tt>-O</tt> option.
Information about limits is available with the <tt>-L</tt> option.

<p>
<B>Learning About the Programs.</B>
After learning how to use a few programs, it
would be a good idea to skim the manual entries to see all the programs and
their options.  Besides the data manipulation and analysis programs, there are
manual entries for special programs included in the |STAT distribution.
<tt>cat</tt> is provided for MSDOS versions that do not have the corresponding
UNIX program.  The MSDOS <tt>type</tt> utility does not handle multiple files
nor wildcards; <tt>cat</tt> does both.  <tt>ff</tt> is a versatile text
formatting filter that allows control of text filling to any width, right
justification, line spacing, pagination, line numbering, tab expansion, and so
on.  <tt>fpack</tt> creates a plain text archive of a series of files.
<tt>fpack</tt> can save space by reducing space wasted by many small files, and
it can save time in file transfers by sending several files in one package.

<p>
<B>Reading Manual Entries Online.</B>
The <tt>manstat</tt> program lets you read the
manual entries online, assuming that they have been installed.  To read the
entry on a program, say <tt>desc</tt>, you just type:
                             <pre>manstat desc</pre>
<p>
The manual entries are also available on the Web:
	<a href="http://www.acm.org/~perlman/stat/doc/">Manual Entries</a>


