
<h2><a name="4">Chapter 4: Data Manipulation</a></h2>

      <UL>
         <li><a href="#4.1">4.1 Data Generation/Augmentation</a>
         <li><a href="#4.2">4.2 Data Transformation</a>
         <li><a href="#4.3">4.3 Data Formatting</a>
         <li><a href="#4.4">4.4 Data Extraction</a>
         <li><a href="#4.5">4.5 Data Validation</a>
         <li><a href="#4.6">4.6 DM: Tutorial and Manual</a>
      </UL>

All data manipulation programs are introduced, showing some of their options.
Full documentation is in the manual entries.  |STAT data manipulation tools
allow users to generate, transform, format, extract, and validate data.
<tt>dm</tt>, the data manipulator, is the most important tool for use with
other |STAT programs.  

<p>
There are several classes of data manipulation programs.
<ul>
	<li><i>Generation</i>
programs produce more data than their inputs by repeating data, numbering data,
or by creating new data.
	<li><i>Transformation</i> programs allow algebraic
conversion of data.
	<li><i>Formatting</i> programs change the shape or order of
the data.
	<li><i>Extraction</i> programs produce subsets of datasets.
	<li><i>Validation</i> programs check the consistency, data types, and ranges of
data.
</ul>

<h3><a name="4.1">4.1 Data Generation/Augmentation</a></h3>

<p>

<b>repeat:  repeat a string or file</b> <br> <tt>repeat</tt> can repeat strings
or lines in a file as many times as requested.  It helps generate labels for
datasets, or feed a program like <tt>dm</tt> that needs input to produce
output.  The following will repeat the file <tt>data</tt> 10 times.  

<pre>
repeat  -n 10  data</pre>

The following will repeat its input series of 20 numbers 15
times.  

<pre>series  1 20  |  repeat  -n 15</pre>

Strings can be repeated using the
<tt>echo</tt> command.  The following will repeat the string <tt>hello</tt> 100
times.  

<pre>echo hello | repeat -n 100 </pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/repeat.htm"
	title="Link to manual entry on the Web"
	>Manual for repeat</a>

<p>

<b>series:  generate a linear series</b> <br> <tt>series</tt> generates a
linear series of numbers between two values.  By default, its values change by
units, but this can be modified.  The following produces a series of 10
numbers, 1 to 10, one per line.  

<pre>series 1 10</pre>

The following produces the
same series, but in reverse order; the start of the series can be greater than
the end.  

<pre>series 10 1</pre>

Non-integral series can be created by supplying an
optional increment.  

<pre>series 0 1 .1</pre>

produces the series: 

<pre>0  .1  .2 .3  .4  .5  .6  .7  .8  .9  1</pre>

except that each value is on its own line.  The
output from series can be transformed with <tt>dm</tt> to produce other than
linear series.  Here is an exponential series: 

<pre>series 1 10 | dm "exp(x1)"</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/series.htm"
	title="Link to manual entry on the Web"
	>Manual for series</a>


<p>

<b>probdist:  generate random numbers</b> <br> <tt>probdist</tt> can generate
random numbers for several probability distributions.  The following will
generate 100 random numbers from the uniform distribution (between 0 and 1).

<pre>probdist random uniform 100</pre>

This can be transformed using <tt>dm</tt> to
get random numbers with other ranges.  The following will produce 100 random
integers uniformly distributed between 10 and 29.  

<pre>probdist random uniform 100 | dm "floor(x1*20+10)"</pre>

The following generates numbers from a one-
trial binomial distribution with probability 0.5.  

<pre>probdist random uniform 100 | dm "if x1 &gt; .5 then 1 else 0"</pre>

<tt>probdist</tt> also has a
binomial distribution built in, so the following would be equivalent to the
previous example: 

<pre>probdist rand binomial 1 1/2 100</pre>

The random number
generator can be seeded.  The following will seed the random number generator
with 143 and generate 100 normally distributed z values.  

<pre>probdist -s 143 random normal 100</pre>

The seeding option is useful when a random sequence must be
repeated.  The random normal numbers have a mean of 0 and a standard deviation
of 1, so <tt>dm</tt> can help create different random normal distributions.
The following samples a normal distribution with mean 100 and standard
deviation 15.  

<pre>probdist random normal 100  |  dm  "x1*15+100"</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/probdist.htm"
	title="Link to manual entry on the Web"
	>Manual for probdist</a>

<p>

<b>abut:  number lines, recycle files</b> <br> <tt>abut</tt> can number input
lines in files using the <tt>-n</tt> option, or cycle through input files as
many times as is necessary to match the length of longer files.  The latter
case is common in creating input files for programs like <tt>anova</tt> and
<tt>contab</tt>, which have input data tagged with regular patterns of labels.

<pre>
<B>File1     File2     Data</B>
large     easy      12
small     easy      23
          hard      34
          hard      45
                    56
                    67
                    78
                    89
</pre>

For the
above input file configuration, the command 

<pre>abut -nc File1 File2 Data</pre>

would produce the following by recycling the smaller files.  

<pre class=data>
1         large     easy      12
2         small     easy      23
3         large     hard      34
4         small     hard      45
5         large     easy      56
6         small     easy      67
7         large     hard      78
8         small     hard      89
</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/abut.htm"
	title="Link to manual entry on the Web"
	>Manual for abut</a>

<p>

<b>dm:  number lines</b> <br> <tt>dm</tt> can number its input lines with its
special variables <tt>INLINE</tt>, which always contains the input line number,
and <tt>INPUT</tt>, which always contains the current input line.  

<pre>dm INLINE INPUT &lt; data</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/dm.htm"
	title="Link to manual entry on the Web"
	>Manual for dm</a>

<h3><a name="4.2">4.2 Data Transformation</a></h3>

<p>
<b>dm:  conditional algebraic combinations of columns</b> <br> <tt>dm</tt> can
produce algebraic combinations of columns.  The following command reads from
<tt>data</tt> and produces the ratio of columns 2 and 1 with column 3 added on.

<pre>dm  x2/x1+x3  &lt;  data</pre>

Transformations can be based on conditions.  For
example, if <tt>x1</tt>, the value in column 1, in the above example is 0, then
<tt>dm</tt> will exit after producing an error message like: 

<pre>dm: division by zero. input line 12  expr[1].</pre>

To avoid this problem, the following will do
the division only if <tt>x1</tt> is non-zero.  

<pre>dm "if x1 then x2/x1+x3 else 0" &lt; data</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/dm.htm"
	title="Link to manual entry on the Web"
	>Manual for dm</a>

<p>
<b>probdist:  probability/statistic conversion</b> <br> <tt>probdist</tt> can
convert probabilities to distribution statistics and <i>vice versa</i> as seen
in tables at the end of most statistics textbooks.  Many distributions are
supported, including: the normal z, binomial, chi-square, F, and t.  The
following will print the two-tailed probability of an obtained t statistic of
2.5 with 20 degrees of freedom.  

<pre>probdist prob t 20 2.5 0.021234</pre>

Similarly, the following will print the two-tailed probability of an F ratio of
6.25 with 1 and 20 degrees of freedom.  

<pre>probdist prob F 1 20 6.25</pre>
<pre class=output>
0.021234</pre>

These results are the same because of the relationship between the t
and F distributions.  <p> The following prints the critical value (also called
the quantile) in the chi-square distribution with 5 degrees of freedom to
obtain a significance level of .05.  

<pre>probdist crit chisq 5 .05</pre>
<pre class=output>
11.070498</pre>

Both probabilities and critical values in the normal z distribution use the
lower one tail -oo to +oo distribution, so the z value that produces the .05
level is obtained with the following.  

<pre>probdist crit z .05</pre>
<pre class=data>
-1.644854</pre>

The
critical value for the 99th percentile is found with the following.  

<pre>
probdist crit z .99
</pre>
<pre class=output>
2.326348</pre>

Binomial distribution critical values are treated
differently than the other continuous distributions.  For the binomial
distribution based on five trials, and a probability of success of one half,
The critical value for a one-tailed test at the .05 level is: 

<pre>probdist crit binomial 5 1/2 .05</pre>
<pre class=output>
5</pre>

even though the probability of 5 successes is
proportionally much less than .05: 

<pre>probdist prob binomial 5 1/2 5</pre>
<pre class=output>
0.031250</pre>

This is because the binomial distribution is discrete.  Not only are
critical values conservative, sometimes there may be no possible value; there
is no way to get a less probable event than five out of five successes: 

<pre>
probdist crit binomial 5 1/2 .01
</pre>
<pre class=output>
6
</pre>

Here, <tt>probdist</tt> is returning an
impossible value (one with zero probability). 

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/probdist.htm"
	title="Link to manual entry on the Web"
	>Manual for probdist</a>

<p>
<b>ranksort:  convert data to ranks</b> <br> <tt>ranksort</tt> can rank order
data from numerical data columns.  For the input: 

<pre class=data>
1   95  4.3
2   113 5.2
3   89  4.5
4   100 5.0
5   89  4.5
</pre>

<tt>ranksort</tt> would produce: 

<pre class=output>
1   3   1
2   5   5
3   1.5 2.5
4   4   4
5   1.5 2.5
</pre>

The ties in the second and third
columns get the average rank of the values for which they are tied.  Once data
are ranksorted, further ranksorting has no effect.  With rank orders within
columns, rank order statistics (e.g., Spearman rank order correlation, average
group rank) can be computed by parametric programs like <tt>pair</tt> or
<tt>regress</tt>.

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/ranksort.htm"
	title="Link to manual entry on the Web"
	>Manual for ranksort</a>

<h3><a name="4.3">4.3 Data Formatting</a></h3>

<p>
<b>maketrix:  form a matrix format file</b> <br> <tt>maketrix</tt> reads its
data, one whitespace separated string at a time from its free format input, and
produces a multicolumn output.  

<pre>series 1 20 | maketrix 5</pre>

The above
produces a five column output.  

<pre>
1   2   3   4   5
6   7   8   9   10
11  12  13  14  15
16  17  18  19  20
</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/maketrix.htm"
	title="Link to manual entry on the Web"
	>Manual for maketrix</a>

<p>
<b>perm:  permute lines</b> <br> <tt>perm</tt>, with no options, randomizes its
input lines.  It can randomize output from programs like <tt>series</tt>.

<pre>series 1 20 | perm</pre>

A subset of this permutation is a sample without
replacement.  The following is a sample of size 10 from the file <tt>data</tt>.

<pre>perm &lt; data | dm "if INLINE &lt;= 10 then INPUT else EXIT"</pre>


<tt>perm</tt> can be supplied a seed for its random number generator, to
replicate a random permutation.  

<pre>series 1 20 | perm -s 5762 | maketrix 5</pre>

The above produces (with my system's random number generator): 


<pre>
18  7   10  13  2
14  11  19  15  20
1   3   9   6   16
8   17  12  5   4
</pre>

<p> <tt>perm</tt> can also put its
lines in alphabetical or numerical order.  For example, the output from the
previous example could be put into ascending order (according to the first
number on each line) with: 

<pre>series 1 20 | perm -s 5762 | maketrix 5 | perm -n</pre>

This produces: 

<pre>
1   3   9   6   16
8   17  12  5   4
14  11  19  15  20
18  7   10  13  2
</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/perm.htm"
	title="Link to manual entry on the Web"
	>Manual for perm</a>

<p>
<b>dsort:  sort data lines by multiple keys</b> <br> The last example of the
<tt>perm</tt> filter showed how lines can be ordered according to the numerical
value in the first column.  <tt>dsort</tt> can sort lines based on numerical or
alphabetical values in any column.  For example, the following command sorts
the previous example matrix in ascending order of the values in the third
column.  

<pre>series 1 20 | perm -s 5762 | maketrix 5 | dsort -n 3</pre>

This
produces: 

<pre>
1   3   9   6   16
18  7   10  13  2
8   17  12  5   4
14  11  19  15  20
</pre>

If there were ties in a column,
<tt>dsort</tt> could sort by additional key columns. 

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/dsort.htm"
	title="Link to manual entry on the Web"
	>Manual for dsort</a>

<p>
<b>transpose:  transpose matrix format file</b> <br> <tt>transpose</tt> flips
rows and columns in its input.  For the input: 

<pre>
1   2   3   4
5   6   7   8
9   10  11  12
</pre>

<tt>transpose</tt> produces: 

<pre>
1   5   9
2   6   10
3   7   11
4   8   12
</pre>

The input to
<tt>transpose</tt> does not have to be regular, nor does it have to be
numerical.  

<pre>
one       two       three
four      five
six
seven     eight
nine      ten       eleven
</pre>

For the above input, <tt>transpose</tt> produces the
following.  

<pre>
one       four      six       seven     nine
two       five                eight     ten
three                                   eleven
</pre>

Note that with regular
inputs, the transposition of a transposition yields the original.  This is not
necessarily so with data as in the above input and output.  The above output
piped through another pass of <tt>transpose</tt> produces a result different
from the original input.  

<pre>
one       two       three
four      five      eleven
six       eight
seven     ten
nine
</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/transpose.htm"
	title="Link to manual entry on the Web"
	>Manual for transpose</a>

<p>
<b>reverse:  reverse lines, columns, characters</b> <br> <tt>reverse</tt> can
reverse the lines, fields, or characters in its input.  It can provide easier
access to the last lines in a file, or the last columns on lines.  To get the
last 10 lines in a file, we can reverse the file, get the first 10 lines, and
then reverse those 10 lines.  

<pre>reverse &lt; data | dm "if INLINE GT 10 then EXIT else INPUT" | reverse</pre>

To get the last two <i>columns</i> in a file is
easier.  

<pre>reverse -f &lt; data | dm s2 s1</pre>

Here, <tt>dm</tt> is used for
column extraction, and rather than call <tt>reverse</tt> a second time, what
were the last two columns before reversal are listed in the opposite order.

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/reverse.htm"
	title="Link to manual entry on the Web"
	>Manual for reverse</a>

<p>
<b>colex:  reorder columns, reformat columns</b> <br> <tt>colex</tt> is a
column extraction program that shares some of the functionality of <tt>dm</tt>
and <tt>reverse</tt>.  <tt>colex</tt> is faster and has a simpler syntax than
<tt>dm</tt> and has data formatting capabilities.  Suppose a matrix dataset
with 10 columns is created with the following.  

<pre>series 1 50 | maketrix 10</pre>

<tt>colex</tt> can extract the last five columns followed by the first five
with the command: 

<pre>series 1 50 | maketrix 10 | colex 6-10 1 2 3 4 5</pre>

Either
ranges of columns or single columns can be given.  The above command produces:

<pre>
6   7   8   9   10  1   2   3   4   5
16  17  18  19  20  11  12  13  14  15
26  27  28  29  30  21  22  23  24  25
36  37  38  39  40  31  32  33  34  35
46  47  48  49  50  41  42  43  44  45
</pre>

<p> Note in the
previous example how the numbers line up on the left, rather than the customary
format to line up the unit digits.  This is because <tt>colex</tt> puts tabs
between columns, and it is not a problem because |STAT programs read data in
free-format.  <tt>colex</tt> can print its columns in several numerical formats
as well as the default string format.  The numerical formatting can round
values to some number of decimal places (like zero, for whole numbers).  The
option: <tt>-F 4i</tt> would tell <tt>colex</tt> to format all the columns as
integers, each four spaces wide, and the <tt>-t</tt> option would tell
<tt>colex</tt> to not place a tab between columns.  The format of columns can
be assigned to individual columns by placing the format before each range of
columns.  For example, the following variation on the previous command would
print columns 6-10 in a money format with two digits after the decimal place,
and columns 1-5 as integers four wide.  

<pre>series 1 50 | maketrix 10 | colex -t 6.2n6-10 4i1-5</pre>

<pre>
  6.00  7.00  8.00  9.00 10.00   1   2   3   4   5
 16.00 17.00 18.00 19.00 20.00  11  12  13  14  15
 26.00 27.00 28.00 29.00 30.00  21  22  23  24  25
 36.00 37.00 38.00 39.00 40.00  31  32  33  34  35
 46.00 47.00 48.00 49.00 50.00  41  42  43  44  45
</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/colex.htm"
	title="Link to manual entry on the Web"
	>Manual for colex</a>

<p>
<b>dm:  reorder columns</b> <br> <tt>dm</tt>, like <tt>colex</tt>, can reorder
columns.  However, it does not allow the specification of ranges of columns.
The above example of <tt>colex</tt> could be done with <tt>dm</tt> with similar
results.  

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/dm.htm"
	title="Link to manual entry on the Web"
	>Manual for dm</a>

<pre>series 1 50 | maketrix 10 | dm s6 s7 s8 s9 s10 s1 s2 s3 s4 s5</pre>
<p>

<b>abut:  paste corresponding lines from files</b> <br> <tt>abut</tt> can join
data in separate files beside one another.  In the usual case, <tt>abut</tt>
takes N files with K lines and produces 1 file with K lines.  Suppose the files
<tt>height</tt> and <tt>weight</tt> contain the respective heights and weights
of the same people.  Each line in each file contains one height or weight.
These could be plotted with the plotting option on the <tt>pair</tt> program
with the following command.  

<pre>abut height weight | pair -p</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/abut.htm"
	title="Link to manual entry on the Web"
	>Manual for abut</a>

<h3><a name="4.4">4.4 Data Extraction</a></h3>

<p>
<b>dm:  conditional data extraction</b> <br> <tt>dm</tt> can extract subsets of
its input, either by columns or by lines.  To extract columns of data, each
extracted column is specified with the number of the column preceded by the
letter <tt>s</tt>.  The following extracts columns 8, 2, and 11, in that order.

<pre>dm s8 s2 s11</pre>

<tt>dm</tt> can extract lines of data by using its built-in
line skipping expression <tt>SKIP</tt>.  The following will extract lines 50 to
100.  

<pre>dm "if INLINE &gt;= 50 &amp; INLINE &lt;= 100 then INPUT else SKIP"</pre>

It is more awkward than column extraction, but the latter is common. 

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/dm.htm"
	title="Link to manual entry on the Web"
	>Manual for dm</a>

<p>
<b>colex:  quick column extraction</b> <br> <tt>colex</tt> can extract
individual columns, or ranges of columns.  For column extraction, it is easier
to use and faster than <tt>dm</tt>.  The following extracts, in order, columns
8, 2, and 11.  

<pre>colex  8  2  11</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/colex.htm"
	title="Link to manual entry on the Web"
	>Manual for colex</a>

<p>
<b>linex:  line extraction</b> <br> <tt>linex</tt> can extract individual lines
(by number), or ranges of lines.  The following extracts, in order, lines 8, 2,
and 11.  

<pre>linex  8  2  11</pre>

To extract lines 50 to 100, you could type:

<pre>linex  50-100</pre>

or you could even extract them in reverse order: 

<pre>
linex  100-50
</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/linex.htm"
	title="Link to manual entry on the Web"
	>Manual for linex</a>

<h3><a name="4.5">4.5 Data Validation</a></h3>

<P>
<b>validata:  data validation</b> <br> <tt>validata</tt> will report for its
input the number of columns, data-types of columns, and for columns with
numerical values, the maxima and minima.  <tt>validata</tt> reports any
inconsistencies in the number of columns in its input.  Floating point numbers
can be entered in scientific notation.  For the input: 

<pre>
1   2   3
4   5   6
7   2E2 end
5       1e-3
</pre>

<tt>validata</tt>'s output is: 

<pre class=output>
validata: Variable number of columns at line 4
Col   N  NA alnum alpha   int float other  type   min   max
  1   4   0     4     0     4     4     0   int     1     7
  2   4   0     3     0     2     4     0 float 0.001   200
  3   3   0     3     1     2     2     0 alnum     3     6
</pre>

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/validata.htm"
	title="Link to manual entry on the Web"
	>Manual for validata</a>

<p>
<b>dm:  conditional data validation</b> <br> <tt>dm</tt> can find exceptional
cases in its input.  A simple case is non-numerical input, which can be checked
with <tt>dm</tt>'s <tt>number</tt> function.  

<pre>dm  "if !number(s1) then 'bad input on line' else SKIP"  INLINE</pre>

<tt>dm</tt> can check for specific
values, ranges of values, or specific relations of values.  The following
prints all lines in <tt>data</tt> with the string <tt>bad</tt> in them.  

<pre>
dm "if 'bad' C INPUT then INPUT else SKIP"
</pre>

The input line number could be
prepended.  

<pre>dm INLINE "if 'bad' C INPUT then INPUT else SKIP"</pre>

This is
possible because <tt>dm</tt> will produce no output for skipped lines,
regardless of expression order.  The following prints all lines where column 3
is greater than column 2.  

<pre>dm "if x3 &gt; x2 then INPUT else SKIP"</pre>

<tt>dm</tt> can print lengths of strings and check for numerical fields: 

<pre>
dm  len(s1)  number(s1)
</pre>

will print the length of column 1 strings, and report
if they are numerical (0 for non-numbers, 1 for integers, 2 for real numbers, 3
for exponential scientific notation numbers).

<p>
<a class=manual href="http://www.acm.org/~perlman/stat/doc/dm.htm"
	title="Link to manual entry on the Web"
	>Manual for dm</a>

